/* Copyright (c) 1997-2012
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.org

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: smith_normal_form.cc 10622 2012-03-04 10:03:59Z paffenho $
*/


#include "polymake/client.h"
#include "polymake/GenericStruct.h"
#include "polymake/Matrix.h"
#include "polymake/GenericMatrix.h"
#include "polymake/SparseMatrix.h"
#include "polymake/Integer.h"
#include "polymake/Smith_normal_form.h"


namespace polymake { namespace common {

    /*
    template <typename Scalar>
    class SmithNormalForm : public GenericStruct<SmithNormalForm<Scalar> > {
    public:
      typedef Matrix<Scalar> matrix_type;
      typedef std::list< std::pair<Scalar,int> > torsion_type;
      DeclSTRUCT( DeclTemplFIELD(SNF,matrix_type)
		  DeclTemplFIELD(LeftCompanion,matrix_type)
		  DeclTemplFIELD(RightCompanion,matrix_type) 
		  DeclTemplFIELD(Torsion,torsion_type)
		  DeclTemplFIELD(Rank,int) );
    };
    */

    class SmithNormalForm : public GenericStruct<SmithNormalForm > {
    public:
      typedef SparseMatrix<Integer> matrix_type;
      typedef std::list< std::pair<Integer,int> > torsion_type;
      DeclSTRUCT( DeclFIELD(SNF,matrix_type)
		  DeclFIELD(LeftCompanion,matrix_type)
		  DeclFIELD(RightCompanion,matrix_type) 
		  DeclFIELD(Torsion,torsion_type)
		  DeclFIELD(Rank,int) );
    };


    /*
    template<typename MatrixTop, typename Scalar>
    typename pm::enable_if<SmithNormalForm<Scalar>, std::numeric_limits<Scalar>::is_integer>::type
    smith_normal_form ( const GenericMatrix<MatrixTop, Scalar>& A, bool inv ) {
    */

    template<typename MatrixTop>
    SmithNormalForm
    smith_normal_form ( const GenericMatrix<MatrixTop, Integer>& A, bool inv ) {

      SparseMatrix<Integer> M(A);
      SparseMatrix<Integer> L,R;
      std::list< std::pair<Integer,int> > torsion;
      int r = 0;
      
      if ( inv )
	r=Smith_normal_form(M,torsion,L,R, pm::False() );
      else
	r=Smith_normal_form(M,torsion,L,R, pm::True() );

      //SmithNormalForm<Scalar> snf;
      SmithNormalForm snf;
      snf.SNF = M;
      snf.LeftCompanion = L;
      snf.RightCompanion = R;
      snf.Torsion = torsion;
      snf.Rank = r;

      return snf;
    }

    PropertyType4perl("SmithNormalForm", SmithNormalForm);

    UserFunctionTemplate4perl("# @category Lattice Polytopes"
			      "# Compute the Smith normal form of a given matrix //M//."
			      "# The function returns a struct SmithNormalForm with five entries: "
			      "# SNF: the Smith normal form S of //M//"
			      "# Torsion: an array containing the absolute values of the entries "
                              "#          greater than 1 of the diagonal together with their multiplicity "
			      "# Rank: the rank of the matrix"
			      "# LeftCompanion/RightCompanion: unimodular matrices L and R such that "
			      "#                               M = LSR if the optional bool value //inv// is false, and "
			      "#                               S = LMR if the optional bool value //inv// is true "
			      "# @param Matrix M, must be of integer type"
			      "# @param Bool inv, optional, if true, compute the inverse of the companion matrices"
			      "# @return SmithNormalForm containing the Smith normal form, the companion matrices, "
                              "#                                    the torsion vector and the rank",
			      "smith_normal_form(Matrix; $=0)");

  }}
