#include "polymake/client.h"

#include "polymake/group/permlib.h"
#include "polymake/Array.h"

namespace polymake { namespace group {

perl::Object construct_symmetric_group(int n){
   perl::Object g("Group");
   Array< Array<int> > sgs(n-1);
   for (int i = 0; i < n-1; ++i) {
      Array<int> gen(n);
      for (int j = 0; j < n; ++j)
         gen[j] = j;
      std::swap(gen[i], gen[i+1]);
      sgs[i] = gen;
   }
   g.take("GENERATORS") << sgs;
   return g;
}

perl::Object construct_cyclic_group(int n){
   perl::Object g("Group");
   Array< Array<int> > sgs(1);
   Array<int> gen(n);
   for (int j = 0; j < n; ++j)
      gen[j] = (j + 1) % n;
   sgs[0] = gen;
   g.take("GENERATORS") << sgs;
   return g;
}


/****************************************************************
user functions
****************************************************************/

UserFunction4perl("# @category Others"
                  "# "
                  "# @param int degree of symmetric group"
                  "# @return Group",
                  &construct_symmetric_group,"construct_symmetric_group($)");


UserFunction4perl("# @category Others"
                  "# "
                  "# @param int degree of cyclic group"
                  "# @return Group",
                  &construct_cyclic_group,"construct_cyclic_group($)");


}
}


