#include "polymake/client.h"
#include "polymake/Rational.h"
#include "polymake/Matrix.h"
#include "polymake/polytope/sympol_interface.h"
#include "polymake/group/permlib.h"

namespace polymake { namespace polytope {
  
   void linear_symmetries(perl::Object p, bool dual)
   {
      group::PermlibGroup sym_group;
      const bool isPolytope = p.isa("polytope::Polytope");
      if (dual) { //V-representation
         sym_group = sympol_interface::sympol_wrapper::compute_linear_symmetries(p.give("VERTICES | RAYS"), Matrix<Rational>());
      } else {
         sym_group = sympol_interface::sympol_wrapper::compute_linear_symmetries(p.give("FACETS"), p.give("AFFINE_HULL | LINEAR_SPAN"));
      }
      const char* groupPoly = "group::GroupOfPolytope";
      const char* groupCone = "group::GroupOfCone";
      const char* correctGroupType = (isPolytope) ? groupPoly : groupCone;
      perl::Object g(correctGroupType);
      g.set_name("linear_symmetries");
      g.take("DOMAIN") << 
         (dual 
            ? polymake::group::OnRays
            : polymake::group::OnFacets
         ); //if dual, then action on vertices(rays), else on facets
      p.add("GROUP", polymake::group::correct_group_from_permlib_group(g, sym_group));
   }
   
   perl::ListReturn representation_conversion_up_to_symmetry(perl::Object p, perl::Object group, bool dual)
   {
      Matrix<Rational> inequalities, equations;
      perl::ListReturn result;
      group::PermlibGroup sym_group = polymake::group::group_from_perlgroup(group);
      bool success = false;
      if (dual) {
         success = sympol_interface::sympol_wrapper::computeFacets(p.give("VERTICES | RAYS"), Matrix<Rational>(), sym_group, dual, inequalities, equations);
      } else {
         success = sympol_interface::sympol_wrapper::computeFacets(p.give("FACETS"), p.give("AFFINE_HULL | LINEAR_SPAN"), sym_group, dual, inequalities, equations);
      }
      result << success << inequalities << equations;
      return result;
   }

UserFunction4perl("# @category Working with symmetries"
                  "# Computes the linear symmetries of a given polytope //p//"
                  "# via 'sympol'. The symmetry group is stored in the property GROUP."
                  "# @param Cone c the cone whose linear symmetry group is to be computed"
                  "# @param bool dual true if group action on vertices, false if action on facets" 
                  "# @return void",
                  &linear_symmetries,"linear_symmetries(Polytope $)");

UserFunction4perl("# @category Working with symmetries"
                  "# Computes the dual description of a polytope up to its linear symmetry group."
                  "# @param Cone c the cone whose dual description is to be computed"
                  "# @param Group a symmetry group of the cone //c// (GroupOfCone or GroupOfPolytope)"
                  "# @param bool dual true if V to H, false if H to V" 
                  "# @return perl::ListReturn list which contains success as bool, vertices/inequalities and lineality/equations as Matrix<Rational>",
                  &representation_conversion_up_to_symmetry,"representation_conversion_up_to_symmetry(Polytope, group::Group $)");
} }

// Local Variables:
// mode:C++
// c-basic-offset:3
// indent-tabs-mode:nil
// End:
