/* Copyright (c) 1997-2010
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: liftings.cc 10623 2012-03-05 16:26:05Z sherrmann $
*/

#include "polymake/tropical/liftings.h"

namespace polymake { namespace tropical {

PuiseuxPolynomial pointlift(const Vector<Rational>& p, const PuiseuxRing& r)
// REMARK: this requires the length of p to match the number of indeterminates
{
  const int n=p.size();
  //  cerr << "pointlift: input = " << p << endl;
  Array< UniPolynomial<Rational,Rational> > coefs(n);
  for (int i=0; i<n; ++i) {
     coefs[i]=UniMonomial<Rational,Rational>(p[i]);
  }
  //  cerr << "pointlift: coefs = " << coefs << endl;
  //  cerr << "pointlift: result = " << PuiseuxPolynomial(unit_matrix<int>(n),coefs,r) << endl;
  return PuiseuxPolynomial(unit_matrix<int>(n),coefs,r);
}


TropicalPolynomial tropicalize(const PuiseuxPolynomial& p)
{
   const int n_terms=p.n_terms();
   Vector<Rational> exps(n_terms);
   Vector<Rational>::iterator e=exps.begin();
   for (Entire<PuiseuxPolynomial::term_hash>::const_iterator i=entire(p.get_terms()); !i.at_end(); ++i, ++e)
      *e=i->second.lm_exp();

   //   cerr << "tropicalize: matrix =\n" << p.monomials_as_matrix< Matrix<int> >() << endl;
   //   cerr << "tropicalize: exps = " << exps << endl;
   return TropicalPolynomial(p.monomials_as_matrix< Matrix<int> >(), exps);
}

} }

// Local Variables:
// mode:C++
// c-basic-offset:3
// indent-tabs-mode:nil
// End:
