#  Copyright (c) 1997-2011
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
#  http://www.polymake.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-------------------------------------------------------------------------------
#  $Project: polymake $$Id: Tempfile.pm 10503 2011-12-20 15:55:44Z benmuell $

use strict;
use namespaces;

package Polymake::Tempfile;

my $dir;

sub find_dir {
   foreach ($ENV{TMPDIR}, "/tmp", "/var/tmp", $PrivateDir, ".") {
      return $_ if defined($_) && -d $_ && -w _ && -x _;
   }
   die <<'.';
Tempfile:: cannot find a writable working directory;
Please set TMPDIR environment variable to a suitable path.
.
}

sub dir { $dir ||= find_dir; }

sub new {
   $dir ||= find_dir;
   bless \(my $name), shift;
}

my $cnt="aaaa0000";
my $rename_cnt=0;
my ($registered_temp_dir, @to_delete);

sub unique_name { "poly".$$."T".(++$cnt) }

use overload '""' => sub { $ {$_[0]} ||= "$dir/".unique_name; };

sub basename { $ {$_[0]} =~ m|/(poly[^/]+)$|; $1 }
sub rename { my $name="$_[0]"; ++$rename_cnt; $name=~s/T(?=\w+$)/N${rename_cnt}_/; $name }

sub DESTROY {
   my ($self)=@_;
   if (defined($$self)) {
      if (my @files=glob $$self."*") {
	 if ($DebugLevel && $@) {
	    warn_print( "temporary files kept: @files\n" );
	 } else {
	    unlink @files;
	 }
      }
   }
}

sub new_dir {
   shift;
   my $path=dir()."/".unique_name();
   mkdir($path, 0700) or die "could not create temporary directory $path: $!\n";
   push @to_delete, $path;
   unless ($registered_temp_dir) {
      add AtEnd("tempdir", sub {
                   if ($DebugLevel) {
                      warn_print( "preserving temporary directories", map{ (" ", $_) } @to_delete );
                   } else {
                      foreach my $dir (@to_delete) {
                         File::Path::rmtree($dir);
                      }
                   }
                });
      $registered_temp_dir=1;
   }
   $path
}
#################################################################################
package Polymake::OverwriteFile;
use Symbol;

use Polymake::Struct (
   [ new => '$' ],
   [ '$dst' => '#1' ],
   '$tempname',
   [ '$handle' => 'undef' ],
);

sub new {
   my $self=&_new;
   my ($dir)= $self->dst =~ $directory_re;
   $dir .= "/" if $dir;
   $self->tempname=$dir . Tempfile->unique_name;
   open $self->handle, ">", $self->tempname
      or die "can't create a new copy of ", $self->dst, ": $!\n";
   ($self->handle, $self);
}

sub DESTROY {
   my ($self)=@_;
   if (defined(fileno($self->handle))) {
      close($self->handle);
      if ($@) {
	 if ($DebugLevel) {
	    warn_print( "incomplete new copy of ", $self->dst, " kept as ", $self->tempname );
	 } else {
	    unlink $self->tempname;
	 }
      } else {
	 unlink $self->tempname;
	 Carp::croak( "creation of new copy of ", $self->dst, " is not properly completed" );
      }
   } elsif (!rename($self->tempname, $self->dst)) {
      warn_print( "could not rename temporary file ", $self->tempname, " to destination ", $self->dst, ": $!" );
   }
}

1

# Local Variables:
# cperl-indent-level:3
# End:
